<?php

namespace Drupal\sfc\Controller;

use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\sfc\ComponentNameHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Enables projects to quickly define routes for components.
 */
class ComponentController extends ControllerBase {

  /**
   * The component plugin manager.
   *
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected $manager;

  /**
   * ComponentController constructor.
   *
   * @param \Drupal\Component\Plugin\PluginManagerInterface $manager
   *   The component plugin manager.
   */
  public function __construct(PluginManagerInterface $manager) {
    $this->manager = $manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.single_file_component')
    );
  }

  /**
   * Generic controller for rendering components.
   *
   * To use this method, define a route like:
   *
   * my_route:
   *   path: /homepage
   *   defaults:
   *     _controller: \Drupal\sfc\Controller\ComponentController::build
   *     component_id: homepage
   *   requirements:
   *     _access: 'TRUE'
   *
   * Other route parameters will be passed directly into context, and the
   * current request will be available in the "request" key.
   *
   * @param string $component_id
   *   The component ID.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return array
   *   The render array.
   */
  public function build($component_id, Request $request) {
    $instance = $this->manager->createInstance($component_id);
    $context = $request->attributes->all();
    $context['request'] = $request;
    return [
      '#type' => 'inline_template',
      '#template' => '{% include "' . ComponentNameHelper::getTemplateName($instance) . '" %}',
      '#context' => $context,
    ];
  }

}
